'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.overrideEntryPoint = exports.addPlugin = exports.mockModule = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _module = require('module');

var _module2 = _interopRequireDefault(_module);

var _mocks = require('./mocks');

var _plugins = require('./plugins');

var _wipeCache = require('./wipeCache');

var _wipeCache2 = _interopRequireDefault(_wipeCache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var originalLoader = _module2.default._load;

var currentModule = 0;
var isolation = false;
var parentModule = module.parent;

var passBy = [];
var mockedModules = {};

var patternMatch = function patternMatch(fileName) {
    return function (pattern) {
        if (typeof pattern == 'function') {
            return pattern(fileName);
        }
        return fileName.match(pattern);
    };
};

var testPassby = function testPassby(request, module) {
    // was called from test
    if (module === parentModule || module == module.parent) {
        return true;
    }
    // if parent was in passlist - pass everythinh
    var fileName = _module2.default._resolveFilename(request, module);
    var m = module;
    while (m) {
        if (passBy.filter(patternMatch(fileName)).length) {
            return true;
        }
        fileName = m.filename;
        m = m.parent;
    }
    return false;
};

function mockLoader(request, parent, isMain) {
    var baseRequest = _module2.default._resolveFilename(request, parent);

    if (parent === parentModule) {
        delete _module2.default._cache[baseRequest];
        mockedModules[baseRequest] = true;
    }

    var mock = (0, _mocks.getMock)(baseRequest) || (0, _mocks.getMock)(request);

    if (mock && (0, _plugins.shouldMock)(mock, request, parent, parentModule)) {
        // this file fill be not cached, but it`s opener - will. And we have to remeber it
        mockedModules[parent.filename] = true;

        mockedModules[baseRequest] = true;

        if (mock.__MI_overrideBy) {
            if (!mock.__MI_module) {
                Object.defineProperty(mock, '__MI_module', {
                    value: originalLoader(mock.__MI_overrideBy, parent, isMain)
                });
            }
            return mock.__MI_module;
        }

        if (mock.__MI_allowCallThought) {
            if (!mock.__MI_module) {
                Object.defineProperty(mock, '__MI_module', {
                    value: originalLoader(request, parent, isMain)
                });
            }
            return _extends({}, mock.__MI_module, mock, {
                __esModule: mock.__MI_module.__esModule
            });
        }
        return mock;
    }

    if (isolation) {
        if (!testPassby(request, parent)) {
            throw new Error('mockModule: ' + request + ' in not listed as passby modules');
        }
    }

    return originalLoader(request, parent, isMain);
}

/** main **/

/**
 * @name rewiremock
 * @param {String} module name
 * @return {rewiremock}
 */
function mockModule(module) {
    currentModule = (0, _plugins.convertName)(module, parentModule);
    (0, _mocks.resetMock)(currentModule);
    return mockModule;
}

/**
 * @name rewiremock.resolve
 * @param {String} module name
 * @return {String} converted module name
 */
mockModule.resolve = function (module) {
    return (0, _plugins.convertName)(module, parentModule);
};

/** mocks **/

/**
 * Enabled call thought original module
 * @name rewiremock.callThought
 * @return {rewiremock}
 */
mockModule.callThought = function () {
    Object.defineProperty((0, _mocks.getMock)(currentModule), "__MI_allowCallThought", {
        value: true
    });
    return mockModule;
};

/**
 * Setting es6 behavior for a current module
 * @return {rewiremock}
 */
mockModule.es6 = function () {
    Object.defineProperty((0, _mocks.getMock)(currentModule), "__esModule", {
        value: true
    });
    return mockModule;
};

/**
 * Setting es6 behavior for a current module and overriding default export
 * @param stub
 * @return {rewiremock}
 */
mockModule.withDefault = function (stub) {
    (0, _mocks.modifyMock)(currentModule, { default: stub });
    return mockModule.es6();
};

/**
 * Overriding export of a module
 * @param stubs
 * @return {rewiremock}
 */
mockModule.with = function (stubs) {
    (0, _mocks.modifyMock)(currentModule, stubs);
    return mockModule;
};

/**
 * Overriding export of one module by another
 * @param {String} name
 * @return {rewiremock}
 */
mockModule.by = function (name) {
    Object.defineProperty((0, _mocks.getMock)(currentModule), "__MI_overrideBy", {
        value: (0, _plugins.convertName)(name, parentModule)
    });
    return mockModule;
};

/** flags **/

/**
 * Activates module isolation
 */
mockModule.isolation = function () {
    isolation = true;
};

/**
 * Deactivates isolation
 */
mockModule.withoutIsolation = function () {
    isolation = false;
};

/**
 * Adding new passby record
 * @param {String|RegEx|Function} pattern
 */
mockModule.passBy = function (pattern) {
    passBy.push(pattern);
};

var overrideEntryPoint = function overrideEntryPoint(parent) {
    parentModule = parent || module.parent.parent;
};

/** interface **/

/**
 * enabled rewiremock
 */
mockModule.enable = function () {
    _module2.default._load = mockLoader;
    (0, _wipeCache2.default)();
};

/**
 * disabled rewiremock
 */
mockModule.disable = function () {
    _module2.default._load = originalLoader;
    mockModule.withoutIsolation();
    mockModule.flush();
};

/**
 * flushes all active overrides
 */
mockModule.flush = function () {
    (0, _wipeCache2.default)(mockedModules);
    mockedModules = {};
};

/**
 * flushes anything
 */
mockModule.clear = function () {
    (0, _mocks.resetMocks)();
    passBy = [];
    mockModule.withoutIsolation();
    mockModule.flush();
};

delete require.cache[require.resolve(__filename)];

exports.mockModule = mockModule;
exports.addPlugin = _plugins.addPlugin;
exports.overrideEntryPoint = overrideEntryPoint;