'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports._clearPlugins = exports.addPlugin = exports.shouldMock = exports.shouldWipe = exports.convertName = undefined;

var _common = require('./plugins/_common');

var plugins = [];

var convertName = function convertName(fileName, parentModule) {
    return plugins.reduce(function (name, plugin) {
        if (plugin.fileNameTransformer) {
            return plugin.fileNameTransformer(name, parentModule) || name;
        }
        return name;
    }, fileName);
};

var triResult = function triResult(values, defaultValue) {
    if (values.indexOf(_common.NO) >= 0) {
        return false;
    }
    if (values.indexOf(_common.YES) >= 0) {
        return true;
    }
    return defaultValue;
};

var shouldMock = function shouldMock(mock, request, parent, topModule) {
    return triResult(plugins.map(function (plugin) {
        return plugin.shouldMock ? plugin.shouldMock(mock, request, parent, topModule) : _common.PASS;
    }), true);
};

var shouldWipe = function shouldWipe(stubs, moduleName) {
    return triResult(plugins.map(function (plugin) {
        return plugin.wipeCheck ? plugin.wipeCheck(stubs, moduleName) : _common.PASS;
    }), false);
};

var addPlugin = function addPlugin(plugin) {
    plugins.push(plugin);
};

var _clearPlugins = function _clearPlugins() {
    return plugins = [];
};

exports.convertName = convertName;
exports.shouldWipe = shouldWipe;
exports.shouldMock = shouldMock;
exports.addPlugin = addPlugin;
exports._clearPlugins = _clearPlugins;